//
//  TwitterAuthEngine.m
//  SPTwitterEngine
//
//  Created by Sam Page on 30/08/11.
//  Copyright 2011 Sam Page. All rights reserved.
//

#import "TwitterAuthEngine.h"

@implementation TwitterAuthEngine
@synthesize delegate;
@synthesize accessToken;

- (id)init
{
    self = [super init];
    if (self) {
		consumer = [[OAConsumer alloc] initWithKey:OAuthConsumerKey
											secret:OAuthConsumerSecret];
    }
    
    return self;
}

- (id)initWithAccessTokenData:(NSString *)key andSecret:(NSString *)secret {
	
	self = [super init];
	if (self) {

		consumer = [[OAConsumer alloc] initWithKey:OAuthConsumerKey
											secret:OAuthConsumerSecret];
		accessToken = [[OAToken alloc] initWithKey:key secret:secret];
		
		[delegate performSelector:@selector(twitterAuthEngineDidReceiveAccessToken:) withObject:accessToken];
	}
	
	return self;
}

#pragma mark OAuthConsumer methods

- (void)getUnauthorizedRequestToken {
	/* Source: http://code.google.com/p/oauthconsumer/wiki/UsingOAuthConsumer */
	
    NSURL *url = [NSURL URLWithString:@"http://api.twitter.com/oauth/request_token"];
	
    request = [[OAMutableURLRequest alloc] initWithURL:url
											  consumer:consumer
												 token:nil   // we don't have a Token yet
												 realm:nil   // our service provider doesn't specify a realm
									 signatureProvider:nil]; // use the default method, HMAC-SHA1
	
    [request setHTTPMethod:@"POST"];
	
    OADataFetcher *fetcher = [[OADataFetcher alloc] init];
	
    [fetcher fetchDataWithRequest:request
                         delegate:self
                didFinishSelector:@selector(requestTokenTicket:didFinishWithData:)
                  didFailSelector:@selector(requestTokenTicket:didFailWithError:)];
	
	[fetcher release];
}

- (void)requestTokenTicket:(OAServiceTicket *)ticket didFinishWithData:(NSData *)data {
	
	if (ticket.didSucceed) {
		NSString *responseBody = [[NSString alloc] initWithData:data
													   encoding:NSUTF8StringEncoding];
		requestToken = [[OAToken alloc] initWithHTTPResponseBody:responseBody];
		NSString *tokenKey = [requestToken key];
		
		NSURL *authURL = [NSURL URLWithString:
						  [NSString stringWithFormat:@"http://api.twitter.com/oauth/authorize?oauth_token=%@", tokenKey]];
		
		[delegate performSelector:@selector(twitterAuthEngineReceivedRequestURL:) withObject:authURL];
		
		[responseBody release];
	}
}

- (void)requestTokenTicket:(OAServiceTicket *)ticket didFailWithError:(NSError *)error {
	
	NSLog(@"Error: %@", [error localizedDescription]);
}

#pragma mark AuthRequestViewControllerDelegate method

- (void)authRequestDidReceiveRequestToken:(OAToken *)token {
		
    NSURL *url = [NSURL URLWithString:@"https://api.twitter.com/oauth/access_token"];
	
    request = [[OAMutableURLRequest alloc] initWithURL:url
											  consumer:consumer
												 token:token 
												 realm:nil
									 signatureProvider:nil];
	
    [request setHTTPMethod:@"POST"];
	
    OADataFetcher *fetcher = [[OADataFetcher alloc] init];
	
    [fetcher fetchDataWithRequest:request
                         delegate:self
                didFinishSelector:@selector(accessTokenTicket:didFinishWithData:)
                  didFailSelector:@selector(accessTokenTicket:didFailWithError:)];
	
	[fetcher release];
}

- (void)accessTokenTicket:(OAServiceTicket *)ticket didFinishWithData:(NSData *)data {
	
	if (ticket.didSucceed) {
		
		NSString *responseBody = [[NSString alloc] initWithData:data
													   encoding:NSUTF8StringEncoding];
		accessToken = [[OAToken alloc] initWithHTTPResponseBody:responseBody];

		[delegate performSelector:@selector(twitterAuthEngineDidReceiveAccessToken:) withObject:accessToken];
		
		[responseBody release];
	}
}

- (void)dealloc {
	
	[super dealloc];
}


@end
