//
//  TwitterRESTEngine.h
//  SPTwitterEngine
//
//  Created by Sam Page on 30/08/11.
//  Copyright 2011 Sam Page. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "OAuthConsumer.h"
#import "SBJson.h"
#import "Tweet.h"
#import "UserProfile.h"

#define HOME_TIMELINE @"http://api.twitter.com/1/statuses/home_timeline.json"
#define USER_TIMELINE @"http://api.twitter.com/1/statuses/user_timeline.json"
#define PUBLIC_TIMELINE @"http://api.twitter.com/1/statuses/public_timeline.json"
#define USER_DATA @"http://api.twitter.com/1/users/show.json"
#define FOLLOWERS @"https://api.twitter.com/1/followers/ids.json"
#define FOLLOWING @"https://api.twitter.com/1/friends/ids.json"
#define USER_LOOKUP @"http://api.twitter.com/1/users/lookup.json"

@protocol TwitterRestEngineDelegate <NSObject>
@optional
- (void)twitterRESTEngineDidReceiveTweets:(NSMutableArray *)_tweets;
- (void)twitterRESTEngineDidFailWithError:(NSError *)_error;
- (void)twitterRESTEngineDidReceiveUserProfile:(UserProfile *)_profile;
- (void)twitterRESTEngineDidReceiveListOfUsers:(NSMutableArray *)_userList;
@end

@interface TwitterRESTEngine : NSObject {
	
	id delegate;
	SBJsonParser *parser;
	OAMutableURLRequest *request;
	OADataFetcher *fetcher;
	OAToken *accessToken;
	OAConsumer *consumer;
	NSString *URL;
	NSString *URLParams;
}

- (id)initWithAccessToken:(OAToken *)_accessToken 
			   andRESTURL:(NSString *)_URL 
			 andURLParams:(NSString *)_URLParams 
			  andDelegate:(id)_delegate;

- (void)getData;
- (void)newTweetFromData:(NSData *)data;
- (void)handleUserProfileResponse:(NSData *)data;
- (void)newUserListFromData:(NSData *)data;
- (void)newUserProfileFromDictionary:(NSDictionary *)userDict;

@end
