//
//  TwitterStreamEngine.m
//  SPTwitterEngine
//
//  Created by Sam Page on 30/08/11.
//  Copyright 2011 Sam Page. All rights reserved.
//

#import "TwitterStreamEngine.h"
#import "TwitterAuthEngine.h"

@implementation TwitterStreamEngine
@synthesize isConnected;

- (id)initWithAccessToken:(OAToken *)_accessToken 
			 andStreamURL:(NSURL *)_streamURL 
				andFilter:(NSString *)_filter 
			  andDelegate:(id)_delegate
{
    self = [super init];
    if (self) {
		
		delegate = _delegate;
		
		accessToken = _accessToken;
		streamURL = _streamURL;
		filter = _filter;
		
		streamParser = [[SBJsonStreamParser alloc] init];
		streamParser.delegate = self;
		streamParser.supportMultipleDocuments = YES;
		
		consumer = [[OAConsumer alloc] initWithKey:OAuthConsumerKey
											secret:OAuthConsumerSecret];
		
    }
    
    return self;
}

- (void)startTweetStream {
	
	isConnected = YES;
	
	request = [[OAMutableURLRequest alloc] initWithURL:streamURL
											  consumer:consumer
												 token:accessToken   // we don't have a Token yet
												 realm:nil   // our service provider doesn't specify a realm
									 signatureProvider:nil]; // use the default method, HMAC-SHA1
	
    [request setHTTPMethod:@"POST"];
	
	[request setHTTPBodyWithString:[filter encodedTwitterParameterString]];
	
	fetcher = [[OAStreamDataFetcher alloc] init];
	
    [fetcher fetchDataWithRequest:request
                         delegate:self
		   didReceiveDataSelector:@selector(streamDownload:didReceiveData:)
                didFinishSelector:@selector(streamDownload:didFinishWithData:)
                  didFailSelector:@selector(streamDownload:didFailWithError:)];
}

- (void)stopTweetStream {
	
	[fetcher cancelStreamConnection];
}

- (void)streamDownload:(OAServiceTicket *)statusTicket didReceiveData:(NSData *)data {
	
	SBJsonStreamParserStatus status = [streamParser parse:data];
	if (status == SBJsonStreamParserError) {
		NSLog(@"Parser error: %@", streamParser.error);
		
	} else if (status == SBJsonStreamParserWaitingForData) {
//		NSLog(@"Parser waiting for more data");
	}
}

- (void)streamDownload:(OAServiceTicket *)statusTicket didFinishWithData:(NSData *)data {
	
	isConnected = NO;
	
	// If the stream finishes, this usually indicates an error.
	NSLog(@"%@ Connection closed", statusTicket);
	
	NSString *str = [[NSString alloc] initWithData:data encoding:NSASCIIStringEncoding];
	NSLog(@"%@", str);
	[str release];
	
	[delegate twitterStreamEngineDidCloseConnection];
}

- (void)streamDownload:(OAServiceTicket *)statusTicket didFailWithError:(NSError *)error {
	
	NSLog(@"Stream download failed: %@", [error localizedDescription]);
}

#pragma mark SBJsonStreamParserDelegate methods

- (void)parser:(SBJsonStreamParser *)parser foundArray:(NSArray *)array {
	
	NSLog(@"Error: Found array"); // This shouldn't happen

}

- (void)parser:(SBJsonStreamParser *)parser foundObject:(NSDictionary *)dict {
	
	if ([dict objectForKey:@"friends"] != nil)
		return;
	
	Tweet *tweet = [[Tweet alloc] init];
	if ([[dict objectForKey:@"user"] objectForKey:@"screen_name"] != nil)
		[tweet setUsername:[[dict objectForKey:@"user"] objectForKey:@"screen_name"]];
	if ([dict objectForKey:@"text"] != nil)
		[tweet setText:[dict objectForKey:@"text"]];
	if ([[dict objectForKey:@"user"] objectForKey:@"name"] != nil)
		[tweet setRealname:[[dict objectForKey:@"user"] objectForKey:@"name"]];
	if ([[dict objectForKey:@"user"] objectForKey:@"profile_image_url"] != nil)
		[tweet setProfileImageURL:[[dict objectForKey:@"user"] objectForKey:@"profile_image_url"]];
	
	if ([[dict objectForKey:@"geo"] isKindOfClass:[NSDictionary class]]) {
		[tweet setIsGeotagged:TRUE];
		[tweet setLatitude:[[[[dict objectForKey:@"geo"] objectForKey:@"coordinates"] objectAtIndex:0] floatValue]];
		[tweet setLongitude:[[[[dict objectForKey:@"geo"] objectForKey:@"coordinates"] objectAtIndex:1] floatValue]];
		CLLocationCoordinate2D coord = {.latitude = [tweet latitude], .longitude = [tweet longitude]};
		tweet.coordinate = coord;
	}
	
	[delegate performSelector:@selector(twitterStreamEngineDidReceiveTweet:) withObject:tweet];
	[tweet release];
}	 

- (void)dealloc {
	[consumer release];
	[request release];
//	[fetcher release];
	[streamParser release];
	[super dealloc];
}

@end
