//
//  RootViewController.m
//  SPTwitterEngineDemo
//
//  Created by Sam Page on 12/10/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "RootViewController.h"
#import "SPTwitterEngineDemoAppDelegate.h"

@implementation RootViewController


- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
	
	appDelegate = [[UIApplication sharedApplication] delegate];
	tweets = [[NSMutableArray alloc] init];
	
	// Add self as an observer for KVO changes relating to the OAUTH Token
	[appDelegate addObserver:self 
				  forKeyPath:@"OAuthToken" 
					 options:NSKeyValueObservingOptionNew 
					 context:NULL];
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated
{
    [super viewDidDisappear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

#pragma mark Key-Value observing
// to listen for if/when the OAUTH token changes (happens at authentication)

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context {
	
	[self initAndStartTwitterStreamEngine:SAMPLE_STREAM andFilter:@""];
	[self initAndStartTwitterRESTEngine:HOME_TIMELINE andFilter:@""];
}


#pragma mark - SPTwitterEngine methods
- (void)initAndStartTwitterRESTEngine:(NSString *)withURL andFilter:(NSString *)filter {
	
#ifdef DEBUG
	NSLog(@"REST URL: %@%@", withURL, filter);
#endif
	
	if (RESTEngine != nil) {
		[RESTEngine release];
		RESTEngine = nil;
	}
	
	RESTEngine = [[TwitterRESTEngine alloc] initWithAccessToken:[appDelegate OAuthToken]
													 andRESTURL:withURL
												   andURLParams:filter
													andDelegate:self];
	[RESTEngine getData];
}

- (void)initAndStartTwitterStreamEngine:(NSURL *)withURL andFilter:(NSString *)filter {
	
#ifdef DEBUG
	NSLog(@"Stream URL: %@%@", [withURL absoluteString], filter);
#endif
	
	if (streamEngine != nil) {
		[streamEngine stopTweetStream];
		[streamEngine release];
		streamEngine = nil;
	}
	
	streamEngine = [[TwitterStreamEngine alloc] initWithAccessToken:[appDelegate OAuthToken]
													   andStreamURL:withURL
														  andFilter:filter
														andDelegate:self];
	[streamEngine startTweetStream];
}

#pragma mark - TwitterRESTEngineDelegate method
- (void)twitterRESTEngineDidReceiveTweets:(NSMutableArray *)_tweets {	
	
	[tweets addObjectsFromArray:_tweets];
	[self.tableView reloadData];
	
	[self.tableView flashScrollIndicators];
}

#pragma mark - TwitterRESTEngineDelegate method
- (void)twitterStreamEngineDidReceiveTweet:(Tweet *)_tweet {
	
	[tweets addObject:_tweet];
	[self.tableView reloadData];
	
	[self.tableView flashScrollIndicators];
}


#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 2;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return [tweets count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"Cell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier] autorelease];
    }
	
	Tweet *tweet = [tweets objectAtIndex:indexPath.row];
	
	[[cell textLabel] setText:[tweet username]];
	[[cell detailTextLabel] setText:[tweet text]];
    
    return cell;
}

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Navigation logic may go here. Create and push another view controller.
    /*
     DetailViewController *detailViewController = [[DetailViewController alloc] initWithNibName:@"Nib name" bundle:nil];
     // ...
     // Pass the selected object to the new view controller.
     [self.navigationController pushViewController:detailViewController animated:YES];
     [detailViewController release];
     */
}

#pragma mark - Dealloc
- (void)dealloc {
	
	[tweets release];
	[RESTEngine release];
	[streamEngine release];
	[super dealloc];
}

@end
